@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2015 EDF S.A.
+# Copyright (C) 1998-2016 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -114,7 +114,7 @@ class Case(object):
             run_ref = os.path.join(self.__repo, self.label, "runcase")
             self.exe, self.pkg = self.__get_exe(pkg, run_ref)
             self.subdomains = []
-            for d in domains:
+            for d in locals()['domains']:
                 if d['solver'] == self.pkg.code_name:
                     self.subdomains.append(d['domain'])
             self.resu = 'RESU_COUPLING'
@@ -135,11 +135,9 @@ class Case(object):
         runcase = cs_runcase.runcase(run_ref)
 
         if runcase.cmd_name == "code_saturne":
-            from Base.XMLinitialize import XMLinit
             from cs_package import package
             pkg = package(old_pkg.scriptdir)
         elif runcase.cmd_name == "neptune_cfd":
-            from core.XMLinitialize import XMLinit
             from nc_package import package
             pkg = package(old_pkg.scriptdir)
 
@@ -195,7 +193,7 @@ class Case(object):
 
         have_gui = 1
         try:
-            f = file(ref, mode = 'r')
+            f = open(ref, mode = 'r')
         except IOError:
            print("Warning SaturneGUI does not exist: %s\n" % ref)
            have_gui = 0
@@ -215,7 +213,7 @@ class Case(object):
                        lines[i] = 'export PATH="":$PATH\n'
                    else:
                        lines[i] = 'export PATH="' + self.pkg.get_dir('bindir') +'":$PATH\n'
-           f = file(ref, mode = 'w')
+           f = open(ref, mode = 'w')
            f.writelines(lines)
            f.close()
 
@@ -233,7 +231,7 @@ class Case(object):
             ref = os.path.join(destdir, subdir, "runcase")
 
         try:
-            f = file(ref, mode = 'r')
+            f = open(ref, mode = 'r')
         except IOError:
             print("Error: can not open %s\n" % ref)
             sys.exit(1)
@@ -252,7 +250,7 @@ class Case(object):
                     lines[i] = 'export PATH="' + self.pkg.get_dir('bindir') +'":$PATH\n'
                 j = lines[i].find(self.pkg.name)
 
-        f = file(ref, mode = 'w')
+        f = open(ref, mode = 'w')
         f.writelines(lines)
         f.close()
 
@@ -264,7 +262,6 @@ class Case(object):
         # 1) Load the xml file of parameters in order to update it
         #    with the __backwardCompatibility method.
 
-        from Base.XMLengine import Case
         from Pages.ScriptRunningModel import ScriptRunningModel
         from cs_exec_environment import \
             separate_args, get_command_single_value, update_command_single_value, assemble_args, enquote_arg
@@ -275,7 +272,6 @@ class Case(object):
             from core.XMLinitialize import XMLinit
 
         if self.subdomains:
-            self.update_runcase_path(d, self.label, xmlonly)
             cdirs = []
             for d in self.subdomains:
                 cdirs.append(os.path.join(self.label, d))
@@ -290,7 +286,7 @@ class Case(object):
         if self.subdomains:
             ref = os.path.join(self.__repo, self.label, "runcase")
             try:
-                f = file(ref, mode = 'r')
+                f = open(ref, mode = 'r')
                 lines = f.readlines()
                 f.close()
                 for i in range(len(lines)):
@@ -302,7 +298,7 @@ class Case(object):
                     else:
                         if re.search(r'^export PATH=', lines[i]):
                             lines[i] = 'export PATH="' + self.pkg.get_dir('bindir') +'":$PATH\n'
-                f = file(ref, mode = 'w')
+                f = open(ref, mode = 'w')
                 f.writelines(lines)
                 f.close()
             except IOError:
@@ -389,10 +385,10 @@ class Case(object):
         # Read the runcase from the Repository
 
         try:
-            f = file(run_ref, mode = 'r')
+            f = open(run_ref, mode = 'r')
         except:
             try:
-                f = file(run_ref_win, mode = 'r')
+                f = open(run_ref_win, mode = 'r')
             except IOError:
                 print("Error: can not open %s\n" % run_ref)
                 sys.exit(1)
@@ -413,7 +409,7 @@ class Case(object):
         # Write the new runcase
 
         try:
-            f = file(run_new, mode = 'r')
+            f = open(run_new, mode = 'r')
         except IOError:
             print("Error: can not open %s\n" % run_new)
             sys.exit(1)
@@ -438,7 +434,7 @@ class Case(object):
                 elif re.search(r' cd ', lines[i]): # do not switch to batch submit directory
                     lines[i] = '# ' + lines[i]
 
-        f = file(run_new, mode = 'w')
+        f = open(run_new, mode = 'w')
         f.writelines(lines)
         f.close()
 
@@ -524,7 +520,7 @@ class Case(object):
 
         if args != None:
             cmd += (" " + args)
-            l = string.split(args)
+            l = args.split()
             try:
                 i = l.index('--threshold')
                 self.threshold = l[i+1]
@@ -585,7 +581,7 @@ class Case(object):
         # 2. The result directory must be read automatically;
         #    check if there is a single result directory.
         elif rep == "":
-            if len(filter(nodot, os.listdir(result))) == 0:
+            if len(list(filter(nodot, os.listdir(result)))) == 0:
                 msg = "Study %s case %s:\nthere is no result directory in %s\nStop.\n" % \
                     (self.__study, self.label, result)
                 studies.reporting(msg)
@@ -593,7 +589,7 @@ class Case(object):
 
             # if no run_id is specified in the xml file
             # only one result directory allowed in RESU
-            if len(filter(nodot, os.listdir(result))) > 1 and self.run_id == "":
+            if len(list(filter(nodot, os.listdir(result)))) > 1 and self.run_id == "":
                 msg = "Study %s case %s:\nthere are several directories in %s\nStop.\n" % \
                     (self.__study, self.label, result)
                 studies.reporting(msg)
@@ -603,7 +599,7 @@ class Case(object):
             # if no run_id is specified in the xml file
             # the only result directory present in RESU is taken
             if rep == "":
-                rep = filter(nodot, os.listdir(result))[0]
+                rep = list(filter(nodot, os.listdir(result)))[0]
 
             rep_f = os.path.join(result, rep)
             if not os.path.isdir(rep_f):
@@ -762,6 +758,7 @@ class Study(object):
         # Create cases
         os.chdir(self.__dest)
 
+        log_lines = []
         for c in self.Cases:
             if not os.path.isdir(c.label):
                 e = os.path.join(c.pkg.get_dir('bindir'), c.exe)
@@ -769,13 +766,16 @@ class Study(object):
                     os.mkdir(c.label)
                     os.chdir(c.label)
                     refdir = os.path.join(self.__repo, c.label)
+                    retval = 1
                     for node in os.listdir(refdir):
                         ref = os.path.join(self.__repo, c.label, node)
                         if node in c.subdomains:
                             cmd = e + " create --case " + node \
                                   + " --quiet --noref --copy-from " \
                                   + ref
-                            retval, t = run_autovnv_command(cmd, self.__log)
+                            node_retval, t = run_autovnv_command(cmd, self.__log)
+                            # negative retcode is kept
+                            retval = min(node_retval,retval)
                         elif os.path.isdir(ref):
                             shutil.copytree(ref, node, symlinks=True)
                         else:
@@ -787,11 +787,17 @@ class Study(object):
                           + " --quiet --noref --copy-from "    \
                           + os.path.join(self.__repo, c.label)
                     retval, t = run_autovnv_command(cmd, self.__log)
+                if retval == 0:
+                    log_lines += ['    - create case: ' + c.label]
+                else:
+                    log_lines += ['    - create case: %s --> FAILED' % c.label]
             else:
                 print("Warning: the case %s already exists in the destination." % c.label)
 
         os.chdir(repbase)
 
+        return log_lines
+
 
     def getRunDirectories(self):
         list_cases = []
@@ -983,9 +989,9 @@ class Studies(object):
         """
         for l, s in self.studies:
             self.reporting('  o Create study: ' + l)
-            s.createCases()
-            for c in s.getCasesLabel():
-                self.reporting('    - create case: ' + c)
+            log_lines = s.createCases()
+            for line in log_lines:
+                self.reporting(line)
 
 
     def compilation(self):
@@ -1107,6 +1113,12 @@ class Studies(object):
         Stop if you try to make a comparison with a file which does not exist.
         """
         for l, s in self.studies:
+            # reference directory passed in autovnv command line overwrites
+            # destination in all cases (even if compare is defined by a compare
+            # markup with a non empty destination)
+            ref = None
+            if self.__ref:
+                ref = os.path.join(self.__ref, s.label)
             for case in s.Cases:
                 if case.compare == 'on' and case.is_run != "KO":
                     compare, nodes, repo, dest, threshold, args = self.__parser.getCompare(case.node)
@@ -1117,16 +1129,13 @@ class Studies(object):
                                 is_checked = True
                                 if destination == False:
                                     dest[i]= None
-                                case.check_dirs(self, nodes[i], repo[i], dest[i])
+                                case.check_dirs(self, nodes[i], repo[i], dest[i], reference=ref)
                     if not compare or not is_checked:
                         node = None
                         repo = ""
                         dest = ""
                         if destination == False:
                             dest = None
-                        ref = None
-                        if self.__ref:
-                            ref = os.path.join(self.__ref, s.label)
                         case.check_dirs(self, node, repo, dest, reference=ref)
 
 
@@ -1137,6 +1146,12 @@ class Studies(object):
         if self.__compare:
             for l, s in self.studies:
                 self.reporting('  o Compare study: ' + l)
+                # reference directory passed in autovnv command line overwrites
+                # destination in all cases (even if compare is defined by a
+                # compare markup with a non empty destination)
+                ref = None
+                if self.__ref:
+                    ref = os.path.join(self.__ref, s.label)
                 for case in s.Cases:
                     if case.compare == 'on' and case.is_run != "KO":
                         is_compare, nodes, repo, dest, t, args = self.__parser.getCompare(case.node)
@@ -1144,17 +1159,18 @@ class Studies(object):
                             for i in range(len(nodes)):
                                 if is_compare[i]:
                                     case.is_compare = "done"
-                                    self.reporting('    - compare %s (with args: %s)' % (case.label, args[i]))
-                                    case.diff_value += case.runCompare(self, repo[i], dest[i], t[i], args[i])
+                                    diff_value = case.runCompare(self, repo[i], dest[i], t[i], args[i], reference=ref)
+                                    case.diff_value += diff_value
+                                    if diff_value:
+                                        self.reporting('    - compare %s (with args: %s) --> DIFFERENCES FOUND' % (case.label, args[i]))
+                                    else:
+                                        self.reporting('    - compare %s (with args: %s) --> NO DIFFERENCES FOUND' % (case.label, args[i]))
                         if not is_compare or case.is_compare != "done":
                             repo = ""
                             dest = ""
                             t    = None
                             args = None
                             case.is_compare = "done"
-                            ref = None
-                            if self.__ref:
-                                ref = os.path.join(self.__ref, s.label)
                             case.diff_value += case.runCompare(self, repo, dest, t, args, reference=ref)
                             if case.diff_value:
                                 self.reporting('    - compare %s (default mode) --> DIFFERENCES FOUND' % (case.label))
