@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A.
+  Copyright (C) 1998-2016 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,8 +58,10 @@
 #include "cs_mesh.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
+#include "cs_physical_model.h"
 #include "cs_prototypes.h"
 #include "cs_selector.h"
+#include "cs_elec_model.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -76,6 +78,7 @@ BEGIN_C_DECLS
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
+#define LG_MAX 1000
 
 /* debugging switch */
 #define _XML_DEBUG_ 0
@@ -1518,240 +1521,12 @@ _get_joule_model(void)
   return model;
 }
 
-
-/*----------------------------------------------------------------------------
- * Return the value of the choice attribute for darcy gravity vector
- *
- * parameters:
- *   name        <--  name of the property
- *----------------------------------------------------------------------------*/
-
-static int
-_darcy_gravity_vector(const char  *name)
-{
-  double value = 0.;
-  char *path   = NULL;
-
-  path = cs_xpath_init_path();
-  cs_xpath_add_elements(&path, 3,
-                        "thermophysical_models",
-                        "darcy_model",
-                        "gravity");
-  cs_xpath_add_element(&path, "vector");
-  cs_xpath_add_element(&path, name);
-  cs_xpath_add_function_text(&path);
-  cs_gui_get_double(path, &value);
-  BFT_FREE(path);
-
-  return value;
-}
-
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
  * Public Fortran function definitions
  *============================================================================*/
 
-/*-----------------------------------------------------------------------------
- * Predefined physics indicator.
- *
- * Fortran Interface:
- *
- * SUBROUTINE UIPPMO
- * *****************
- *
- * INTEGER          IPPMOD <--  specific physics indicator array
- * INTEGER          ICOD3P  --> diffusion flame in fast complete chemistry
- * INTEGER          ICODEQ  --> diffusion flame in fast chemistry towards balance
- * INTEGER          ICOEBU  --> Eddy Break Up premixing flame
- * INTEGER          ICOBML  --> Bray - Moss - Libby premixing flame
- * INTEGER          ICOLWC  --> Libby Williams premixing flame
- * INTEGER          ICP3PL  --> Coal combustion. Combustible moyen local
- * INTEGER          ICPL3C  --> Coal combustion coupled with lagrangien approach
- * INTEGER          ICFUEL  --> Fuel combustion
- * INTEGER          IELJOU  --> Joule effect
- * INTEGER          IELARC  --> electrical arc
- * INTEGER          IELION  --> ionique mobility
- * INTEGER          ICOMPF  --> compressible without shock
- * INTEGER          IATMOS  --> atmospheric flows
- * INTEGER          IAEROS  --> cooling tower
- * INTEGER          INDJON  --> INDJON=1: a JANAF enthalpy-temperature
- *                              tabulation is used. INDJON=1: users tabulation
- * INTEGER          IEOS    --> compressible
- * INTEGER          IEQCO2  --> CO2 massic fraction transport
- * INTEGER          IDARCY  --> darcy model
- *
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uippmo, UIPPMO)(int *const ippmod,
-                               int *const icod3p,
-                               int *const icodeq,
-                               int *const icoebu,
-                               int *const icobml,
-                               int *const icolwc,
-                               int *const iccoal,
-                               int *const icpl3c,
-                               int *const icfuel,
-                               int *const ieljou,
-                               int *const ielarc,
-                               int *const ielion,
-                               int *const icompf,
-                               int *const iatmos,
-                               int *const iaeros,
-                               int *const ieos,
-                               int *const ieqco2,
-                               int *const idarcy)
-{
-  int isactiv = 0;
-
-  cs_var_t  *vars = cs_glob_var;
-
-  ippmod[*icod3p - 1] = -1;
-  ippmod[*icodeq - 1] = -1;
-  ippmod[*icoebu - 1] = -1;
-  ippmod[*icobml - 1] = -1;
-  ippmod[*icolwc - 1] = -1;
-  ippmod[*iccoal - 1] = -1;
-  ippmod[*icpl3c - 1] = -1;
-  ippmod[*icfuel - 1] = -1;
-  ippmod[*ieljou - 1] = -1;
-  ippmod[*ielarc - 1] = -1;
-  ippmod[*ielion - 1] = -1;
-  ippmod[*icompf - 1] = -1;
-  ippmod[*iatmos - 1] = -1;
-  ippmod[*iaeros - 1] = -1;
-  ippmod[*idarcy - 1] = -1;
-
-  *ieqco2 = 0;
-
-  /* Look for the active specific physics and give the value of the associated
-     model attribute */
-  isactiv = cs_gui_get_activ_thermophysical_model();
-
-  if (isactiv)
-  {
-    if (cs_gui_strcmp(vars->model, "solid_fuels"))
-    {
-      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel"))
-        ippmod[*iccoal - 1] = 0;
-      else if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel_moisture") || cs_gui_strcmp(vars->model_value, "homogeneous_fuel_moisture_lagr"))
-        ippmod[*iccoal - 1] = 1;
-      else
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Invalid coal model: %s.\n"), vars->model_value);
-      // ieqco2 fix to transport of CO2 mass fraction
-      *ieqco2 = 1;
-    }
-    else if  (cs_gui_strcmp(vars->model, "gas_combustion"))
-    {
-      char *path = NULL;
-      path = cs_xpath_init_path();
-      cs_xpath_add_elements(&path, 2, "thermophysical_models", "gas_combustion");
-      cs_xpath_add_attribute(&path, "model");
-
-      char *model = cs_gui_get_attribute_value(path);
-
-      BFT_FREE(path);
-
-      if (!cs_gui_strcmp(model, "off")) {
-
-        if (cs_gui_strcmp(vars->model_value, "adiabatic"))
-          ippmod[*icod3p - 1] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "extended"))
-          ippmod[*icod3p - 1] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "spalding"))
-          ippmod[*icoebu - 1] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "enthalpy_st"))
-          ippmod[*icoebu - 1] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "mixture_st"))
-          ippmod[*icoebu - 1] = 2;
-        else if (cs_gui_strcmp(vars->model_value, "enthalpy_mixture_st"))
-          ippmod[*icoebu - 1] = 3;
-        else if (cs_gui_strcmp(vars->model_value, "2-peak_adiabatic"))
-          ippmod[*icolwc - 1] = 0;
-        else if (cs_gui_strcmp(vars->model_value, "2-peak_enthalpy"))
-          ippmod[*icolwc - 1] = 1;
-        else if (cs_gui_strcmp(vars->model_value, "3-peak_adiabatic"))
-          ippmod[*icolwc - 1] = 2;
-        else if (cs_gui_strcmp(vars->model_value, "3-peak_enthalpy"))
-          ippmod[*icolwc - 1] = 3;
-        else if (cs_gui_strcmp(vars->model_value, "4-peak_adiabatic"))
-          ippmod[*icolwc - 1] = 4;
-        else if (cs_gui_strcmp(vars->model_value, "4-peak_enthalpy"))
-          ippmod[*icolwc - 1] = 5;
-        else
-          bft_error(__FILE__, __LINE__, 0,
-                    _("Invalid gas combustion flow model: %s.\n"),
-                    vars->model_value);
-      }
-    }
-    else if  (cs_gui_strcmp(vars->model, "atmospheric_flows"))
-    {
-      if (cs_gui_strcmp(vars->model_value, "constant"))
-        ippmod[*iatmos - 1] = 0;
-      else if (cs_gui_strcmp(vars->model_value, "dry"))
-        ippmod[*iatmos - 1] = 1;
-      else if (cs_gui_strcmp(vars->model_value, "humid"))
-        ippmod[*iatmos - 1] = 2;
-      else
-        bft_error(__FILE__, __LINE__, 0,
-            _("Invalid atmospheric flow model: %s.\n"),
-            vars->model_value);
-    }
-    else if  (cs_gui_strcmp(vars->model, "joule_effect"))
-    {
-      if (cs_gui_strcmp(vars->model_value, "joule"))
-      {
-        char *value = _get_joule_model();
-        if (cs_gui_strcmp(value, "AC/DC"))
-          ippmod[*ieljou - 1] = 1;
-        else if (cs_gui_strcmp(value, "three-phase"))
-          ippmod[*ieljou - 1] = 2;
-        else if (cs_gui_strcmp(value, "AC/DC+Transformer"))
-          ippmod[*ieljou - 1] = 3;
-        else if (cs_gui_strcmp(value, "three-phase+Transformer"))
-          ippmod[*ieljou - 1] = 4;
-        else
-          bft_error(__FILE__, __LINE__, 0,
-              _("Invalid joule model: %s.\n"),
-              vars->model_value);
-        BFT_FREE(value);
-      }
-      else if (cs_gui_strcmp(vars->model_value, "arc"))
-        ippmod[*ielarc - 1] = 2;
-      else
-        bft_error(__FILE__, __LINE__, 0,
-            _("Invalid electrical model: %s.\n"),
-            vars->model_value);
-    }
-    else if  (cs_gui_strcmp(vars->model, "compressible_model"))
-    {
-      if (cs_gui_strcmp(vars->model_value, "constant_gamma")){
-        ippmod[*icompf - 1] = 0;
-        *ieos = 1;
-      }
-      else
-        bft_error(__FILE__, __LINE__, 0,
-            _("Invalid compressible model: %s.\n"),
-            vars->model_value);
-    }
-    else if  (cs_gui_strcmp(vars->model, "darcy_model"))
-    {
-      if (cs_gui_strcmp(vars->model_value, "darcy"))
-        ippmod[*idarcy - 1] = 1;
-    }
-  }
-
-#if _XML_DEBUG_
-  bft_printf("==>UIPPMO\n");
-  if (isactiv)
-  {
-    bft_printf("--thermophysical model: %s\n", vars->model);
-    bft_printf("--thermophysical value: %s\n", vars->model_value);
-  }
-#endif
-}
-
 /*----------------------------------------------------------------------------
  * Density under relaxation
  *
@@ -1808,238 +1583,6 @@ void CS_PROCF (uicpi2, UICPI2) (double *const toxy,
 }
 
 
-/*----------------------------------------------------------------------------
- * Electrical model : read parameters
- *
- * Fortran Interface:
- *
- * subroutine uieli1
- * *****************
- * integer         ieljou    -->   joule model
- * integer         ielarc    -->   arc model
- * integer         ielcor    <--   scaling electrical variables
- * double          couimp    <--   imposed current intensity
- * double          puisim    <--   imposed power
- * integer         modrec    <--   scaling type for electric arc
- * integer         idreca    <--   current density component used to scaling
- *                                 (modrec ==2)
- * char            crit_reca <--   define criteria for plane used to scaling (modrec ==2)
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uieli1, UIELI1) (const int    *const ieljou,
-                                const int    *const ielarc,
-                                      int    *const ielcor,
-                                      double *const couimp,
-                                      double *const puisim,
-                                      int    *const modrec,
-                                      int    *const idreca,
-                                      double *const crit_reca)
-{
-  char *path   = NULL;
-  int   status = 0;
-  double   result;
-
-  path = cs_xpath_init_path();
-  cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "joule_effect",
-                                  "variable_scaling");
-
-  cs_xpath_add_attribute(&path, "status");
-  if (cs_gui_get_status(path, &status))
-    *ielcor = status;
-  BFT_FREE(path);
-
-  if (*ieljou > 0) {
-    path = cs_xpath_init_path();
-    cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                    "joule_effect",
-                                    "imposed_power");
-
-    cs_xpath_add_function_text(&path);
-    if (!cs_gui_get_double(path, &result))
-      bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-    *puisim = result;
-    BFT_FREE(path);
-  }
-
-  if (*ielarc > 0) {
-    path = cs_xpath_init_path();
-    cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                    "joule_effect",
-                                    "imposed_current");
-    cs_xpath_add_function_text(&path);
-    if (!cs_gui_get_double(path, &result))
-      bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-    *couimp = result;
-    BFT_FREE(path);
-
-    if (*ielcor > 0) {
-      path = cs_xpath_init_path();
-      char *choice;
-
-      cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                      "joule_effect",
-                                      "recal_model");
-      cs_xpath_add_attribute(&path, "model");
-      choice = cs_gui_get_attribute_value(path);
-      if (cs_gui_strcmp(choice, "general_case"))
-        *modrec = 1;
-      else if (cs_gui_strcmp(choice, "plane_define"))
-        *modrec = 2;
-      else if (cs_gui_strcmp(choice, "user"))
-        *modrec = 3;
-      else
-        bft_error(__FILE__, __LINE__, 0, _("Invalid model : %s\n"), choice);
-
-      BFT_FREE(choice);
-
-      if (*modrec == 2) {
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 4, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "direction");
-        cs_xpath_add_function_text(&path);
-        choice = cs_gui_get_text_value(path);
-        if (cs_gui_strcmp(choice, "X"))
-          *idreca = 1;
-        else if (cs_gui_strcmp(choice, "Y"))
-          *idreca = 2;
-        else
-          *idreca = 3;
-        BFT_FREE(path);
-        BFT_FREE(choice);
-
-        double val;
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 5, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "plane_definition",
-                                        "A");
-        cs_xpath_add_function_text(&path);
-        if (!cs_gui_get_double(path, &val))
-          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-        BFT_FREE(path);
-        crit_reca[0] = val;
-
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 5, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "plane_definition",
-                                        "B");
-        cs_xpath_add_function_text(&path);
-        if (!cs_gui_get_double(path, &val))
-          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-        BFT_FREE(path);
-        crit_reca[1] = val;
-
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 5, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "plane_definition",
-                                        "C");
-        cs_xpath_add_function_text(&path);
-        if (!cs_gui_get_double(path, &val))
-          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-        BFT_FREE(path);
-        crit_reca[2] = val;
-
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 5, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "plane_definition",
-                                        "D");
-        cs_xpath_add_function_text(&path);
-        if (!cs_gui_get_double(path, &val))
-          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-        BFT_FREE(path);
-        crit_reca[3] = val;
-
-        path = cs_xpath_init_path();
-        cs_xpath_add_elements(&path, 5, "thermophysical_models",
-                                        "joule_effect",
-                                        "recal_model",
-                                        "plane_definition",
-                                        "epsilon");
-        cs_xpath_add_function_text(&path);
-        if (!cs_gui_get_double(path, &val))
-          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
-        BFT_FREE(path);
-        crit_reca[4] = val;
-      }
-    }
-    BFT_FREE(path);
-  }
-
-#if _XML_DEBUG_
-  bft_printf("==>UIELI1\n");
-  bft_printf("--ielcor  = %i\n", *ielcor);
-  bft_printf("--puisim  = %f\n", *puisim);
-  bft_printf("--couimp  = %f\n", *couimp);
-  bft_printf("--modrec  = %f\n", *modrec);
-#endif
-}
-
-/*----------------------------------------------------------------------------
- * Electrical model : define plane for elreca
- *
- * Fortran Interface:
- *
- * subroutine uielrc
- * *****************
- * integer         izreca    <--   define plane used to scaling (modrec ==2)
- * char            crit_reca <--   define criteria for plane used to scaling (modrec ==2)
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uielrc, UIELRC) (int    *const izreca,
-                                double *const crit_reca)
-{
-  /* build list of cells */
-  char *crit = NULL;
-
-  cs_lnum_t   n_selected_faces = 0;
-  cs_lnum_t  *selected_faces = NULL;
-
-  BFT_MALLOC(crit, 66, char);
-
-  char cVal[10];
-
-  strcpy(crit, "plane[");
-  sprintf(cVal, "%f", crit_reca[0]);
-  strcat(crit, cVal);
-  strcat(crit, ",");
-  sprintf(cVal, "%f", crit_reca[1]);
-  strcat(crit, cVal);
-  strcat(crit, ",");
-  sprintf(cVal, "%f", crit_reca[2]);
-  strcat(crit, cVal);
-  strcat(crit, ",");
-  sprintf(cVal, "%f", crit_reca[3]);
-  strcat(crit, cVal);
-  strcat(crit, ",epsilon=");
-  sprintf(cVal, "%6f", crit_reca[4]);
-  strcat(crit, cVal);
-  strcat(crit, "]");
-
-  BFT_MALLOC(selected_faces, cs_glob_mesh->n_i_faces, cs_lnum_t);
-
-  cs_selector_get_i_face_list(crit,
-                              &n_selected_faces,
-                              selected_faces);
-
-  for (int j=0; j < n_selected_faces; j++)
-    izreca[selected_faces[j]] = 1;
-
-  BFT_FREE(selected_faces);
-  BFT_FREE(crit);
-
-}
-
-
 /*----------------------------------------------------------------------------
  * Atmospheric flows: read of meteorological file of data
  *
@@ -2122,10 +1665,7 @@ void CS_PROCF (uiati1, UIATI1) (int           *imeteo,
  * (pulverized solid fuels)
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uisofu, UISOFU) (const int    *const ippmod,
-                                const int    *const iccoal,
-                                const int    *const icpl3c,
-                                const int    *const iirayo,
+void CS_PROCF (uisofu, UISOFU) (const int    *const iirayo,
                                 const int    *const iihmpr,
                                 const int    *const ncharm,
                                 int          *const ncharb,
@@ -2550,32 +2090,22 @@ void CS_PROCF(cfnmtd, CFNMTD) (char          *fstr,    /* --> Fortran string */
 
 
 /*----------------------------------------------------------------------------
- * darcy model : read parameters
+ * groundwater model : read parameters
  *
  * Fortran Interface:
  *
  * subroutine uidai1
  * *****************
- * integer         iricha          -->   darcy model
  * integer         permeability    <--   permeability type
- * integer         diffusion       <--   diffusion type
+ * integer         dispersion      <--   dispersion type
  * integer         unsteady        <--   steady flow
- * integer         convergence     <--   convergence criterion of Newton scheme
  * integer         gravity         <--   check if gravity is taken into account
- * double          gravity_x       <--   x component for gravity vector
- * double          gravity_y       <--   y component for gravity vector
- * double          gravity_z       <--   z component for gravity vector
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uidai1, UIDAI1) (const int    *const idarcy,
-                                      int    *const permeability,
-                                      int    *const diffusion,
-                                      int    *const unsteady,
-                                      int    *const convergence,
-                                      int    *const gravity,
-                                      double *gravity_x,
-                                      double *gravity_y,
-                                      double *gravity_z)
+void CS_PROCF (uidai1, UIDAI1) (int    *const permeability,
+                                int    *const dispersion,
+                                int    *const unsteady,
+                                int    *const gravity)
 {
   char *path   = NULL;
   char *mdl    = NULL;
@@ -2583,39 +2113,23 @@ void CS_PROCF (uidai1, UIDAI1) (const int    *const idarcy,
 
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "darcy_model",
-                                  "criterion");
-
-  cs_xpath_add_attribute(&path, "model");
-  mdl = cs_gui_get_attribute_value(path);
-  BFT_FREE(path);
-  if (cs_gui_strcmp(mdl, "pressure"))
-    *convergence = 0;
-  else
-    *convergence = 1;
-
-  BFT_FREE(mdl);
-  BFT_FREE(path);
-
-  path = cs_xpath_init_path();
-  cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "darcy_model",
-                                  "diffusion");
+                                  "groundwater_model",
+                                  "dispersion");
 
   cs_xpath_add_attribute(&path, "model");
   mdl = cs_gui_get_attribute_value(path);
   BFT_FREE(path);
   if (cs_gui_strcmp(mdl, "anisotropic"))
-    *diffusion = 1;
+    *dispersion = 1;
   else
-    *diffusion = 0;
+    *dispersion = 0;
 
   BFT_FREE(mdl);
   BFT_FREE(path);
 
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "darcy_model",
+                                  "groundwater_model",
                                   "flowType");
 
   cs_xpath_add_attribute(&path, "model");
@@ -2631,7 +2145,7 @@ void CS_PROCF (uidai1, UIDAI1) (const int    *const idarcy,
 
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "darcy_model",
+                                  "groundwater_model",
                                   "permeability");
 
   cs_xpath_add_attribute(&path, "model");
@@ -2647,7 +2161,7 @@ void CS_PROCF (uidai1, UIDAI1) (const int    *const idarcy,
 
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
-                                  "darcy_model",
+                                  "groundwater_model",
                                   "gravity");
   cs_xpath_add_attribute(&path, "status");
 
@@ -2656,33 +2170,372 @@ void CS_PROCF (uidai1, UIDAI1) (const int    *const idarcy,
 
   BFT_FREE(path);
 
-  if (*gravity == 1) {
-    double rotation_axis[3];
-    rotation_axis[0] = _darcy_gravity_vector("axis_x");
-    rotation_axis[1] = _darcy_gravity_vector("axis_y");
-    rotation_axis[2] = _darcy_gravity_vector("axis_z");
-    double len = sqrt(rotation_axis[0]*rotation_axis[0] +
-                      rotation_axis[1]*rotation_axis[1] +
-                      rotation_axis[2]*rotation_axis[2]);
-    *gravity_x = rotation_axis[0] / len;
-    *gravity_y = rotation_axis[1] / len;
-    *gravity_z = rotation_axis[2] / len;
-  }
-
 #if _XML_DEBUG_
   bft_printf("==>UIDAI1\n");
-  bft_printf("--darcy_anisotropic_permeability  = %i\n", *permeability);
-  bft_printf("--darcy_anisotropic_diffusion     = %f\n", *diffusion);
-  bft_printf("--darcy_unsteady                  = %f\n", *unsteady);
-  bft_printf("--darcy_convergence_criterion     = %f\n", *convergence);
+  bft_printf("--groundwater_anisotropic_permeability  = %i\n", *permeability);
+  bft_printf("--groundwater_anisotropic_dispersion    = %f\n", *dispersion);
+  bft_printf("--groundwater_unsteady                  = %f\n", *unsteady);
 #endif
 }
 
-
 /*============================================================================
  * Public function definitions
  *============================================================================*/
 
+/*-----------------------------------------------------------------------------
+ * Activate specific physical models based on XML settings.
+ *
+ * parameters:
+ *   ieos    --> compressible
+ *   ieqco2  --> CO2 massic fraction transport (for combustion only)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_physical_model_select(cs_int_t  *ieos,
+                             cs_int_t  *ieqco2)
+{
+  if (!cs_gui_file_is_loaded())
+    return;
+
+  int isactiv = 0;
+
+  cs_var_t  *vars = cs_glob_var;
+
+  *ieqco2 = 0;
+
+  /* Look for the active specific physics and give the value of the associated
+     model attribute */
+  isactiv = cs_gui_get_activ_thermophysical_model();
+
+  if (isactiv)  {
+
+    if (cs_gui_strcmp(vars->model, "solid_fuels")) {
+      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel"))
+        cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 0;
+      else if (   cs_gui_strcmp(vars->model_value,
+                                "homogeneous_fuel_moisture")
+               || cs_gui_strcmp(vars->model_value,
+                                "homogeneous_fuel_moisture_lagr"))
+        cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 1;
+      else
+        bft_error(__FILE__, __LINE__, 0,
+                  _("Invalid coal model: %s.\n"), vars->model_value);
+      /* ieqco2 fix to transport of CO2 mass fraction */
+      *ieqco2 = 1;
+    }
+    else if  (cs_gui_strcmp(vars->model, "gas_combustion")) {
+      char *path = NULL;
+      path = cs_xpath_init_path();
+      cs_xpath_add_elements(&path, 2, "thermophysical_models", "gas_combustion");
+      cs_xpath_add_attribute(&path, "model");
+
+      char *model = cs_gui_get_attribute_value(path);
+
+      BFT_FREE(path);
+
+      if (!cs_gui_strcmp(model, "off")) {
+
+        if (cs_gui_strcmp(vars->model_value, "adiabatic"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_3PT] = 0;
+        else if (cs_gui_strcmp(vars->model_value, "extended"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_3PT] = 1;
+        else if (cs_gui_strcmp(vars->model_value, "spalding"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 0;
+        else if (cs_gui_strcmp(vars->model_value, "enthalpy_st"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 1;
+        else if (cs_gui_strcmp(vars->model_value, "mixture_st"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 2;
+        else if (cs_gui_strcmp(vars->model_value, "enthalpy_mixture_st"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_EBU] = 3;
+        else if (cs_gui_strcmp(vars->model_value, "2-peak_adiabatic"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 0;
+        else if (cs_gui_strcmp(vars->model_value, "2-peak_enthalpy"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 1;
+        else if (cs_gui_strcmp(vars->model_value, "3-peak_adiabatic"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 2;
+        else if (cs_gui_strcmp(vars->model_value, "3-peak_enthalpy"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 3;
+        else if (cs_gui_strcmp(vars->model_value, "4-peak_adiabatic"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 4;
+        else if (cs_gui_strcmp(vars->model_value, "4-peak_enthalpy"))
+          cs_glob_physical_model_flag[CS_COMBUSTION_LW] = 5;
+        else
+          bft_error(__FILE__, __LINE__, 0,
+                    _("Invalid gas combustion flow model: %s.\n"),
+                    vars->model_value);
+      }
+    }
+    else if (cs_gui_strcmp(vars->model, "atmospheric_flows")) {
+      if (cs_gui_strcmp(vars->model_value, "constant"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 0;
+      else if (cs_gui_strcmp(vars->model_value, "dry"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 1;
+      else if (cs_gui_strcmp(vars->model_value, "humid"))
+        cs_glob_physical_model_flag[CS_ATMOSPHERIC] = 2;
+      else
+        bft_error(__FILE__, __LINE__, 0,
+            _("Invalid atmospheric flow model: %s.\n"),
+            vars->model_value);
+    }
+    else if (cs_gui_strcmp(vars->model, "joule_effect")) {
+      if (cs_gui_strcmp(vars->model_value, "joule")) {
+        char *value = _get_joule_model();
+        if (cs_gui_strcmp(value, "AC/DC"))
+          cs_glob_physical_model_flag[CS_JOULE_EFFECT] = 1;
+        else if (cs_gui_strcmp(value, "three-phase"))
+          cs_glob_physical_model_flag[CS_JOULE_EFFECT] = 2;
+        else if (cs_gui_strcmp(value, "AC/DC+Transformer"))
+          cs_glob_physical_model_flag[CS_JOULE_EFFECT] = 3;
+        else if (cs_gui_strcmp(value, "three-phase+Transformer"))
+          cs_glob_physical_model_flag[CS_JOULE_EFFECT] = 4;
+        else
+          bft_error(__FILE__, __LINE__, 0,
+              _("Invalid joule model: %s.\n"),
+              vars->model_value);
+        BFT_FREE(value);
+      }
+      else if (cs_gui_strcmp(vars->model_value, "arc"))
+        cs_glob_physical_model_flag[CS_ELECTRIC_ARCS] = 2;
+      else
+        bft_error(__FILE__, __LINE__, 0,
+            _("Invalid electrical model: %s.\n"),
+            vars->model_value);
+    }
+    else if (cs_gui_strcmp(vars->model, "compressible_model")) {
+      if (cs_gui_strcmp(vars->model_value, "constant_gamma")) {
+        cs_glob_physical_model_flag[CS_COMPRESSIBLE] = 0;
+        *ieos = 1;
+      }
+      else
+        bft_error(__FILE__, __LINE__, 0,
+            _("Invalid compressible model: %s.\n"),
+            vars->model_value);
+    }
+    else if (cs_gui_strcmp(vars->model, "groundwater_model")) {
+      if (cs_gui_strcmp(vars->model_value, "groundwater"))
+        cs_glob_physical_model_flag[CS_GROUNDWATER] = 1;
+    }
+  }
+
+#if _XML_DEBUG_
+  bft_printf("%s\n", __func__);
+  if (isactiv) {
+    bft_printf("--thermophysical model: %s\n", vars->model);
+    bft_printf("--thermophysical value: %s\n", vars->model_value);
+  }
+#endif
+}
+
+/*----------------------------------------------------------------------------
+ * Electrical model: read parameters
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_elec_model(void)
+{
+  char *path   = NULL;
+  int   status = 0;
+  double   result;
+
+  path = cs_xpath_init_path();
+  cs_xpath_add_elements(&path, 3, "thermophysical_models",
+                                  "joule_effect",
+                                  "variable_scaling");
+
+  cs_elec_option_t *elec_opt = cs_get_glob_elec_option();
+
+  cs_xpath_add_attribute(&path, "status");
+  if (cs_gui_get_status(path, &status))
+    elec_opt->ielcor = status;
+  BFT_FREE(path);
+
+  if (cs_glob_elec_option->ieljou > 0) {
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 3, "thermophysical_models",
+                                    "joule_effect",
+                                    "imposed_power");
+
+    cs_xpath_add_function_text(&path);
+    if (!cs_gui_get_double(path, &result))
+      bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+    elec_opt->puisim = result;
+    BFT_FREE(path);
+  }
+
+  if (cs_glob_elec_option->ielarc > 0) {
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 3, "thermophysical_models",
+                                    "joule_effect",
+                                    "imposed_current");
+    cs_xpath_add_function_text(&path);
+    if (!cs_gui_get_double(path, &result))
+      bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+    elec_opt->couimp = result;
+    BFT_FREE(path);
+
+    if (cs_glob_elec_option->ielcor > 0) {
+      path = cs_xpath_init_path();
+      char *choice;
+
+      cs_xpath_add_elements(&path, 3, "thermophysical_models",
+                                      "joule_effect",
+                                      "recal_model");
+      cs_xpath_add_attribute(&path, "model");
+      choice = cs_gui_get_attribute_value(path);
+      if (cs_gui_strcmp(choice, "general_case"))
+        elec_opt->modrec = 1;
+      else if (cs_gui_strcmp(choice, "plane_define"))
+        elec_opt->modrec = 2;
+      else if (cs_gui_strcmp(choice, "user"))
+        elec_opt->modrec = 3;
+      else
+        bft_error(__FILE__, __LINE__, 0, _("Invalid model : %s\n"), choice);
+
+      BFT_FREE(choice);
+
+      if (cs_glob_elec_option->modrec == 2) {
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 4, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "direction");
+        cs_xpath_add_function_text(&path);
+        choice = cs_gui_get_text_value(path);
+        if (cs_gui_strcmp(choice, "X"))
+          elec_opt->idreca = 1;
+        else if (cs_gui_strcmp(choice, "Y"))
+          elec_opt->idreca = 2;
+        else
+          elec_opt->idreca = 3;
+        BFT_FREE(path);
+        BFT_FREE(choice);
+
+        double val;
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 5, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "plane_definition",
+                                        "A");
+        cs_xpath_add_function_text(&path);
+        if (!cs_gui_get_double(path, &val))
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+        BFT_FREE(path);
+        elec_opt->crit_reca[0] = val;
+
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 5, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "plane_definition",
+                                        "B");
+        cs_xpath_add_function_text(&path);
+        if (!cs_gui_get_double(path, &val))
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+        BFT_FREE(path);
+        elec_opt->crit_reca[1] = val;
+
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 5, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "plane_definition",
+                                        "C");
+        cs_xpath_add_function_text(&path);
+        if (!cs_gui_get_double(path, &val))
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+        BFT_FREE(path);
+        elec_opt->crit_reca[2] = val;
+
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 5, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "plane_definition",
+                                        "D");
+        cs_xpath_add_function_text(&path);
+        if (!cs_gui_get_double(path, &val))
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+        BFT_FREE(path);
+        elec_opt->crit_reca[3] = val;
+
+        path = cs_xpath_init_path();
+        cs_xpath_add_elements(&path, 5, "thermophysical_models",
+                                        "joule_effect",
+                                        "recal_model",
+                                        "plane_definition",
+                                        "epsilon");
+        cs_xpath_add_function_text(&path);
+        if (!cs_gui_get_double(path, &val))
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+        BFT_FREE(path);
+        elec_opt->crit_reca[4] = val;
+      }
+    }
+    BFT_FREE(path);
+  }
+
+#if _XML_DEBUG_
+  bft_printf("==>UIELI1\n");
+  bft_printf("--ielcor  = %i\n", cs_glob_elec_option->ielcor);
+  bft_printf("--puisim  = %f\n", cs_glob_elec_option->puisim);
+  bft_printf("--couimp  = %f\n", cs_glob_elec_option->couimp);
+  bft_printf("--modrec  = %f\n", cs_glob_elec_option->modrec);
+#endif
+}
+
+/*----------------------------------------------------------------------------
+ * Electrical model: define plane for elreca
+ *
+ * Fortran Interface:
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_elec_model_rec(void)
+{
+  /* build list of cells */
+  char *crit = NULL;
+
+  cs_lnum_t   n_selected_faces = 0;
+  cs_lnum_t  *selected_faces = NULL;
+
+  BFT_MALLOC(crit, 66, char);
+
+  char cVal[10];
+
+  cs_elec_option_t *elec_opt = cs_get_glob_elec_option();
+
+  strcpy(crit, "plane[");
+  sprintf(cVal, "%f", elec_opt->crit_reca[0]);
+  strcat(crit, cVal);
+  strcat(crit, ",");
+  sprintf(cVal, "%f", elec_opt->crit_reca[1]);
+  strcat(crit, cVal);
+  strcat(crit, ",");
+  sprintf(cVal, "%f", elec_opt->crit_reca[2]);
+  strcat(crit, cVal);
+  strcat(crit, ",");
+  sprintf(cVal, "%f", elec_opt->crit_reca[3]);
+  strcat(crit, cVal);
+  strcat(crit, ",epsilon=");
+  sprintf(cVal, "%6f", elec_opt->crit_reca[4]);
+  strcat(crit, cVal);
+  strcat(crit, "]");
+
+  BFT_MALLOC(selected_faces, cs_glob_mesh->n_i_faces, cs_lnum_t);
+
+  cs_selector_get_i_face_list(crit,
+                              &n_selected_faces,
+                              selected_faces);
+
+  for (int j=0; j < n_selected_faces; j++)
+    elec_opt->izreca[selected_faces[j]] = 1;
+
+  BFT_FREE(selected_faces);
+  BFT_FREE(crit);
+
+}
+
 /*-----------------------------------------------------------------------------
  * Return the name of a thermophysical model.
  *
@@ -2729,14 +2582,14 @@ cs_gui_get_activ_thermophysical_model(void)
                          "joule_effect",
                          "atmospheric_flows",
                          "compressible_model",
-                         "darcy_model" };
+                         "groundwater_model" };
   int name_nbr = sizeof(name) / sizeof(name[0]);
 
   if (vars->model != NULL && vars->model_value != NULL) {
     isactiv = 1;
     return isactiv;
   } else {
-    vars->model = NULL;
+    BFT_FREE(vars->model);
     vars->model_value = NULL;
   }
 
